"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeviceMessageHandler = void 0;
const error_1 = require("../error");
const utils_1 = require("../utils");
const command_1 = require("./command");
const event_1 = require("./event");
const properties_1 = require("./properties");
class DeviceMessageHandler {
    static getStreamingDevices(stationSN) {
        if (DeviceMessageHandler.streamingDevices[stationSN] !== undefined)
            return DeviceMessageHandler.streamingDevices[stationSN];
        return [];
    }
    static removeStreamingDevice(stationSN, client) {
        if (DeviceMessageHandler.streamingDevices[stationSN] !== undefined)
            DeviceMessageHandler.streamingDevices[stationSN] = DeviceMessageHandler.streamingDevices[stationSN].filter((cl) => cl !== client);
    }
    static addStreamingDevice(stationSN, client) {
        if (DeviceMessageHandler.streamingDevices[stationSN] !== undefined && !DeviceMessageHandler.streamingDevices[stationSN].includes(client))
            DeviceMessageHandler.streamingDevices[stationSN].push(client);
        else if (DeviceMessageHandler.streamingDevices[stationSN] === undefined)
            DeviceMessageHandler.streamingDevices[stationSN] = [client];
    }
    static getDownloadingDevices(stationSN) {
        if (DeviceMessageHandler.downloadingDevices[stationSN] !== undefined)
            return DeviceMessageHandler.downloadingDevices[stationSN];
        return [];
    }
    static removeDownloadingDevice(stationSN, client) {
        if (DeviceMessageHandler.downloadingDevices[stationSN] !== undefined)
            DeviceMessageHandler.downloadingDevices[stationSN] = DeviceMessageHandler.downloadingDevices[stationSN].filter((cl) => cl !== client);
    }
    static addDownloadingDevice(stationSN, client) {
        if (DeviceMessageHandler.downloadingDevices[stationSN] !== undefined && !DeviceMessageHandler.downloadingDevices[stationSN].includes(client))
            DeviceMessageHandler.downloadingDevices[stationSN].push(client);
        else if (DeviceMessageHandler.downloadingDevices[stationSN] === undefined)
            DeviceMessageHandler.downloadingDevices[stationSN] = [client];
    }
    static getTalkbackingDevices(stationSN) {
        if (DeviceMessageHandler.talkbackingDevices[stationSN] !== undefined)
            return DeviceMessageHandler.talkbackingDevices[stationSN];
        return [];
    }
    static removeTalkbackingDevice(stationSN, client) {
        if (DeviceMessageHandler.talkbackingDevices[stationSN] !== undefined)
            DeviceMessageHandler.talkbackingDevices[stationSN] = DeviceMessageHandler.talkbackingDevices[stationSN].filter((cl) => cl !== client);
    }
    static addTalkbackingDevice(stationSN, client) {
        if (DeviceMessageHandler.talkbackingDevices[stationSN] !== undefined && !DeviceMessageHandler.talkbackingDevices[stationSN].includes(client))
            DeviceMessageHandler.talkbackingDevices[stationSN].push(client);
        else if (DeviceMessageHandler.talkbackingDevices[stationSN] === undefined)
            DeviceMessageHandler.talkbackingDevices[stationSN] = [client];
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    static async handle(message, driver, client) {
        const { serialNumber, command } = message;
        const device = await driver.getDevice(serialNumber);
        const station = await driver.getStation(device.getStationSerial());
        switch (command) {
            case command_1.DeviceCommand.setStatusLed:
                if (client.schemaVersion <= 12) {
                    await station.setStatusLed(device, message.value).catch((error) => {
                        throw error;
                    });
                    return {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.setAutoNightVision:
                if (client.schemaVersion <= 12) {
                    await station.setAutoNightVision(device, message.value).catch((error) => {
                        throw error;
                    });
                    return {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.setMotionDetection:
                if (client.schemaVersion <= 12) {
                    await station.setMotionDetection(device, message.value).catch((error) => {
                        throw error;
                    });
                    return {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.setSoundDetection:
                if (client.schemaVersion <= 12) {
                    await station.setSoundDetection(device, message.value).catch((error) => {
                        throw error;
                    });
                    return {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.setPetDetection:
                if (client.schemaVersion <= 12) {
                    await station.setPetDetection(device, message.value).catch((error) => {
                        throw error;
                    });
                    return {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.setRTSPStream:
                if (client.schemaVersion <= 12) {
                    await station.setRTSPStream(device, message.value).catch((error) => {
                        throw error;
                    });
                    return {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.setAntiTheftDetection:
                if (client.schemaVersion <= 12) {
                    await station.setAntiTheftDetection(device, message.value).catch((error) => {
                        throw error;
                    });
                    return {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.setWatermark:
                if (client.schemaVersion <= 12) {
                    await station.setWatermark(device, message.value).catch((error) => {
                        throw error;
                    });
                    return {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.enableDevice:
                if (client.schemaVersion <= 12) {
                    await station.enableDevice(device, message.value).catch((error) => {
                        throw error;
                    });
                    return {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.lockDevice:
                if (client.schemaVersion <= 12) {
                    await station.lockDevice(device, message.value).catch((error) => {
                        throw error;
                    });
                    return {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.getPropertiesMetadata:
                {
                    const properties = device.getPropertiesMetadata();
                    if (client.schemaVersion <= 3) {
                        return { properties: properties };
                    }
                    else if (client.schemaVersion >= 4 && client.schemaVersion <= 12) {
                        return {
                            serialNumber: device.getSerial(),
                            properties: properties
                        };
                    }
                    else {
                        return {
                            serialNumber: device.getSerial(),
                            properties: (0, properties_1.dumpDevicePropertiesMetadata)(device, client.schemaVersion)
                        };
                    }
                }
            case command_1.DeviceCommand.getProperties:
                {
                    const properties = device.getProperties();
                    if (client.schemaVersion <= 3) {
                        return { properties: properties };
                    }
                    else if (client.schemaVersion >= 4 && client.schemaVersion <= 12) {
                        return {
                            serialNumber: device.getSerial(),
                            properties: properties
                        };
                    }
                    else {
                        return {
                            serialNumber: device.getSerial(),
                            properties: (0, properties_1.dumpDeviceProperties)(device, client.schemaVersion)
                        };
                    }
                }
            case command_1.DeviceCommand.setProperty:
                await driver.setDeviceProperty(serialNumber, message.name, message.value).catch((error) => {
                    throw error;
                });
                return client.schemaVersion >= 13 ? { async: true } : {};
            case command_1.DeviceCommand.startLivestream:
                if (client.schemaVersion >= 2) {
                    if (!station.isLiveStreaming(device)) {
                        await station.startLivestream(device).catch((error) => {
                            throw error;
                        });
                        client.receiveLivestream[serialNumber] = true;
                        DeviceMessageHandler.addStreamingDevice(station.getSerial(), client);
                    }
                    else if (client.receiveLivestream[serialNumber] !== true) {
                        //TODO: Cache last received I-Frame and send that with the rest to the new client
                        client.sendEvent({
                            source: "device",
                            event: event_1.DeviceEvent.livestreamStarted,
                            serialNumber: serialNumber,
                        });
                        client.receiveLivestream[serialNumber] = true;
                        DeviceMessageHandler.addStreamingDevice(station.getSerial(), client);
                    }
                    else {
                        throw new error_1.LivestreamAlreadyRunningError(`Livestream for device ${serialNumber} is already running`);
                    }
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.stopLivestream:
                if (client.schemaVersion >= 2) {
                    if (!station.isLiveStreaming(device)) {
                        throw new error_1.LivestreamNotRunningError(`Livestream for device ${serialNumber} could not be stopped, because it is not running`);
                    }
                    if (client.receiveLivestream[serialNumber] !== true) {
                        throw new error_1.LivestreamNotRunningError(`This client has not requested the start of the live stream for the device ${serialNumber} and therefore cannot request its termination`);
                    }
                    if (DeviceMessageHandler.streamingDevices[station.getSerial()] !== undefined && DeviceMessageHandler.streamingDevices[station.getSerial()].includes(client)) {
                        if (DeviceMessageHandler.streamingDevices[station.getSerial()].length === 1) {
                            DeviceMessageHandler.removeStreamingDevice(station.getSerial(), client);
                            await station.stopLivestream(device).catch((error) => {
                                throw error;
                            });
                        }
                        else {
                            client.receiveLivestream[serialNumber] = false;
                            DeviceMessageHandler.removeStreamingDevice(station.getSerial(), client);
                            client.sendEvent({
                                source: "device",
                                event: event_1.DeviceEvent.livestreamStopped,
                                serialNumber: serialNumber,
                            });
                        }
                    }
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.isLiveStreaming:
                {
                    if (client.schemaVersion >= 2) {
                        const result = station.isLiveStreaming(device);
                        if (client.schemaVersion >= 2 && client.schemaVersion <= 3) {
                            return { livestreaming: result };
                        }
                        else if (client.schemaVersion >= 4) {
                            return {
                                serialNumber: device.getSerial(),
                                livestreaming: result
                            };
                        }
                    }
                    else {
                        throw new error_1.UnknownCommandError(command);
                    }
                }
            case command_1.DeviceCommand.triggerAlarm:
                if (client.schemaVersion >= 3) {
                    await station.triggerDeviceAlarmSound(device, message.seconds).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.resetAlarm:
                if (client.schemaVersion >= 3) {
                    await station.resetDeviceAlarmSound(device).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.panAndTilt:
                if (client.schemaVersion >= 3) {
                    await station.panAndTilt(device, message.direction).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.quickResponse:
                if (client.schemaVersion >= 3) {
                    await station.quickResponse(device, message.voiceId).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.startDownload:
                if (client.schemaVersion >= 3) {
                    if (!station.isDownloading(device)) {
                        await station.startDownload(device, message.path, message.cipherId).catch((error) => {
                            throw error;
                        });
                        client.receiveDownloadStream[serialNumber] = true;
                        DeviceMessageHandler.addDownloadingDevice(station.getSerial(), client);
                    }
                    else if (client.receiveDownloadStream[serialNumber] !== true) {
                        throw new error_1.DownloadOnlyOneAtATimeError(`Download for device ${serialNumber} is already running from another client`);
                    }
                    else {
                        throw new error_1.DownloadAlreadyRunningError(`Download for device ${serialNumber} is already running`);
                    }
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.cancelDownload:
                if (client.schemaVersion >= 3) {
                    if (!station.isDownloading(device)) {
                        throw new error_1.DownloadNotRunningError(`Download for device ${serialNumber} could not be cancelled, because it is not running`);
                    }
                    if (client.receiveDownloadStream[serialNumber] !== true) {
                        throw new error_1.DownloadNotRunningError(`This client has not requested the start of the download for the device ${serialNumber} and therefore cannot request its cancellation`);
                    }
                    if (DeviceMessageHandler.downloadingDevices[station.getSerial()] !== undefined && DeviceMessageHandler.downloadingDevices[station.getSerial()].includes(client)) {
                        if (DeviceMessageHandler.downloadingDevices[station.getSerial()].length === 1) {
                            DeviceMessageHandler.removeDownloadingDevice(station.getSerial(), client);
                            await station.cancelDownload(device).catch((error) => {
                                throw error;
                            });
                        }
                    }
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.isDownloading:
                {
                    if (client.schemaVersion >= 13) {
                        const result = station.isDownloading(device);
                        return {
                            serialNumber: device.getSerial(),
                            downloading: result
                        };
                    }
                    else {
                        throw new error_1.UnknownCommandError(command);
                    }
                }
            case command_1.DeviceCommand.getVoices:
                if (client.schemaVersion >= 3) {
                    const voices = await driver.getApi().getVoices(device.getSerial()).catch((error) => {
                        throw error;
                    });
                    if (client.schemaVersion === 3) {
                        return {
                            serialNumber: device.getSerial(),
                            voices: voices
                        };
                    }
                    else if (client.schemaVersion >= 4) {
                        return {
                            serialNumber: device.getSerial(),
                            voices: voices
                        };
                    }
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.hasProperty:
                {
                    if (client.schemaVersion >= 3) {
                        const result = device.hasProperty(message.propertyName);
                        if (client.schemaVersion === 3) {
                            return { exists: result };
                        }
                        else if (client.schemaVersion >= 4) {
                            return {
                                serialNumber: device.getSerial(),
                                exists: result
                            };
                        }
                    }
                    else {
                        throw new error_1.UnknownCommandError(command);
                    }
                }
            case command_1.DeviceCommand.hasCommand:
                {
                    if (client.schemaVersion >= 3) {
                        const result = device.hasCommand(message.commandName);
                        if (client.schemaVersion === 3) {
                            return { exists: result };
                        }
                        else if (client.schemaVersion >= 4) {
                            return {
                                serialNumber: device.getSerial(),
                                exists: result
                            };
                        }
                    }
                    else {
                        throw new error_1.UnknownCommandError(command);
                    }
                }
            case command_1.DeviceCommand.getCommands:
                {
                    if (client.schemaVersion >= 3) {
                        const result = device.getCommands();
                        const commands = [];
                        result.forEach(command => {
                            commands.push((0, utils_1.convertCamelCaseToSnakeCase)(command.replace("device", "")));
                        });
                        if (client.schemaVersion === 3) {
                            return { commands: result };
                        }
                        else if (client.schemaVersion >= 4) {
                            return {
                                serialNumber: device.getSerial(),
                                commands: commands //TODO: Convert to correct type
                            };
                        }
                    }
                    else {
                        throw new error_1.UnknownCommandError(command);
                    }
                }
            case command_1.DeviceCommand.startRTSPLivestream:
                if (client.schemaVersion >= 6) {
                    await station.startRTSPStream(device).catch((error) => {
                        throw error;
                    });
                    return {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.stopRTSPLivestream:
                if (client.schemaVersion >= 6) {
                    await station.stopRTSPStream(device).catch((error) => {
                        throw error;
                    });
                    return {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.isRTSPLiveStreaming:
                {
                    if (client.schemaVersion >= 6) {
                        const result = station.isRTSPLiveStreaming(device);
                        return {
                            serialNumber: device.getSerial(),
                            livestreaming: result
                        };
                    }
                    else {
                        throw new error_1.UnknownCommandError(command);
                    }
                }
            case command_1.DeviceCommand.calibrateLock:
                if (client.schemaVersion >= 9) {
                    await station.calibrateLock(device).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.calibrate:
                if (client.schemaVersion >= 10) {
                    await station.calibrate(device).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.setDefaultAngle:
                if (client.schemaVersion >= 10) {
                    await station.setDefaultAngle(device).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.setPrivacyAngle:
                if (client.schemaVersion >= 10) {
                    await station.setPrivacyAngle(device).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.unlock:
                if (client.schemaVersion >= 13) {
                    await station.unlock(device).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.startTalkback:
                if (client.schemaVersion >= 13) {
                    if (!station.isTalkbackOngoing(device)) {
                        await station.startTalkback(device).catch((error) => {
                            throw error;
                        });
                        client.sendTalkbackStream[serialNumber] = true;
                        DeviceMessageHandler.addTalkbackingDevice(station.getSerial(), client);
                    }
                    else if (client.sendTalkbackStream[serialNumber] !== true) {
                        throw new error_1.TalkbackOnlyOneAtATimeError(`Talkback for device ${serialNumber} is already running from another client`);
                    }
                    else {
                        throw new error_1.TalkbackAlreadyRunningError(`Talkback for device ${serialNumber} is already running`);
                    }
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.stopTalkback:
                if (client.schemaVersion >= 13) {
                    if (!station.isTalkbackOngoing(device)) {
                        throw new error_1.TalkbackNotRunningError(`Talkback for device ${serialNumber} could not be cancelled, because it is not running`);
                    }
                    if (client.sendTalkbackStream[serialNumber] !== true) {
                        throw new error_1.TalkbackNotRunningError(`This client has not requested the start of the talkback for the device ${serialNumber} and therefore cannot request its termination`);
                    }
                    if (DeviceMessageHandler.talkbackingDevices[station.getSerial()] !== undefined && DeviceMessageHandler.talkbackingDevices[station.getSerial()].includes(client)) {
                        if (DeviceMessageHandler.talkbackingDevices[station.getSerial()].length === 1) {
                            DeviceMessageHandler.talkbackStream?.end();
                            DeviceMessageHandler.talkbackStream = undefined;
                            DeviceMessageHandler.removeTalkbackingDevice(station.getSerial(), client);
                            await station.stopTalkback(device).catch((error) => {
                                throw error;
                            });
                        }
                    }
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.isTalkbackOngoing:
                {
                    if (client.schemaVersion >= 13) {
                        const result = station.isTalkbackOngoing(device);
                        return {
                            serialNumber: device.getSerial(),
                            talkbackOngoing: result
                        };
                    }
                    else {
                        throw new error_1.UnknownCommandError(command);
                    }
                }
            case command_1.DeviceCommand.talkbackAudioData:
                if (client.schemaVersion >= 13) {
                    if (!station.isTalkbackOngoing(device)) {
                        throw new error_1.TalkbackNotRunningError(`Talkback for device ${serialNumber} could not be cancelled, because it is not running`);
                    }
                    if (client.sendTalkbackStream[serialNumber] !== true) {
                        throw new error_1.TalkbackNotRunningError(`This client has not requested the start of the talkback for the device ${serialNumber} and therefore cannot send audio data`);
                    }
                    if (DeviceMessageHandler.talkbackingDevices[station.getSerial()] !== undefined && DeviceMessageHandler.talkbackingDevices[station.getSerial()].includes(client)) {
                        const audioDataBuffer = Buffer.from(message.buffer);
                        this.talkbackStream?.write(audioDataBuffer);
                    }
                    return {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.snooze:
                if (client.schemaVersion >= 13) {
                    await station.snooze(device, {
                        snooze_time: message.snoozeTime,
                        snooze_chime: message.snoozeChime,
                        snooze_motion: message.snoozeMotion,
                        snooze_homebase: message.snoozeHomebase,
                    }).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.addUser:
                if (client.schemaVersion >= 13) {
                    const addUser = message;
                    await driver.addUser(device.getSerial(), addUser.username, addUser.passcode, addUser.schedule).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.deleteUser:
                if (client.schemaVersion >= 13) {
                    const addUser = message;
                    await driver.deleteUser(device.getSerial(), addUser.username).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.getUsers:
                if (client.schemaVersion >= 13) {
                    const users = await driver.getApi().getUsers(device.getSerial(), device.getStationSerial()).catch((error) => {
                        throw error;
                    });
                    return { users: users === null ? [] : users };
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.updateUser:
                if (client.schemaVersion >= 13) {
                    const addUser = message;
                    await driver.updateUser(device.getSerial(), addUser.username, addUser.newUsername).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.updateUserPasscode:
                if (client.schemaVersion >= 13) {
                    const addUser = message;
                    await driver.updateUserPasscode(device.getSerial(), addUser.username, addUser.passcode).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.updateUserSchedule:
                if (client.schemaVersion >= 13) {
                    const addUser = message;
                    await driver.updateUserSchedule(device.getSerial(), addUser.username, addUser.schedule).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.DeviceCommand.verifyPIN:
                if (client.schemaVersion >= 13) {
                    const verifyPIN = message;
                    await station.verifyPIN(device, verifyPIN.pin).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            default:
                throw new error_1.UnknownCommandError(command);
        }
    }
}
exports.DeviceMessageHandler = DeviceMessageHandler;
DeviceMessageHandler.streamingDevices = {};
DeviceMessageHandler.downloadingDevices = {};
DeviceMessageHandler.talkbackingDevices = {};
//# sourceMappingURL=message_handler.js.map