"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.waitForEvent = exports.convertCamelCaseToSnakeCase = void 0;
const convertCamelCaseToSnakeCase = function (value) {
    return value !== undefined ? value.replace(/[A-Z]/g, (letter, index) => {
        return index == 0 ? letter.toLowerCase() : "_" + letter.toLowerCase();
    }) : "";
};
exports.convertCamelCaseToSnakeCase = convertCamelCaseToSnakeCase;
const waitForEvent = function (emitter, event, timeout) {
    return new Promise((resolve, reject) => {
        let internalTimeout = undefined;
        const success = (val) => {
            emitter.off("error", fail);
            resolve(val);
        };
        const fail = (err) => {
            emitter.off(event, success);
            reject(err);
        };
        emitter.once(event, success);
        emitter.once("error", fail);
        if (internalTimeout)
            clearTimeout(internalTimeout);
        if (timeout) {
            internalTimeout = setTimeout(() => {
                emitter.off(event, success);
                emitter.off("error", fail);
                reject(new Error("Timeout reached"));
            }, timeout);
        }
    });
};
exports.waitForEvent = waitForEvent;
//# sourceMappingURL=utils.js.map